from fastapi import APIRouter, HTTPException, Body
from terratest.core.image_manager import ImageManager
from terratest.constants import DEFAULT_TERRAFORM_IMAGE
from terratest.utils.config_store import get_config_store

router = APIRouter()

@router.get("/status")
def get_image_status():
    """Verifica si la imagen Docker existe y obtiene su información."""
    # Obtener imagen configurada o usar la por defecto
    config_store = get_config_store()
    selected_image = config_store.get('selected_docker_image', DEFAULT_TERRAFORM_IMAGE)
    
    manager = ImageManager(selected_image)
    
    exists = manager.check_image_exists()
    info = manager.get_image_info() if exists else None
    dockerfile_path = manager.get_dockerfile_path()
    
    return {
        "exists": exists,
        "image_name": selected_image,
        "default_image": DEFAULT_TERRAFORM_IMAGE,
        "info": info,
        "dockerfile_found": dockerfile_path is not None,
        "dockerfile_path": str(dockerfile_path) if dockerfile_path else None
    }

@router.get("/list")
def list_all_images():
    """Lista todas las imágenes Docker disponibles localmente."""
    manager = ImageManager()
    images = manager.list_all_images()
    
    # Obtener imagen seleccionada actualmente
    config_store = get_config_store()
    selected_image = config_store.get('selected_docker_image', DEFAULT_TERRAFORM_IMAGE)
    
    return {
        "images": images,
        "selected_image": selected_image,
        "default_image": DEFAULT_TERRAFORM_IMAGE
    }

@router.post("/select")
def select_image(image_name: str = Body(..., embed=True)):
    """Selecciona una imagen Docker existente para usar."""
    manager = ImageManager(image_name)
    
    # Verificar que la imagen existe
    if not manager.check_image_exists():
        raise HTTPException(status_code=404, detail=f"Imagen '{image_name}' no encontrada")
    
    # Guardar en config store
    config_store = get_config_store()
    config_store.set('selected_docker_image', image_name)
    
    return {
        "status": "success",
        "message": f"Imagen '{image_name}' seleccionada",
        "image_name": image_name
    }

@router.post("/build")
def build_image(dockerfile_content: str = Body(None, embed=True)):
    """
    Construye la imagen Docker de Terraform.
    
    Args:
        dockerfile_content: Contenido opcional del Dockerfile como string.
                          Si no se proporciona, se busca en el filesystem o se usa uno por defecto.
    """
    manager = ImageManager(DEFAULT_TERRAFORM_IMAGE)
    
    # Verificar si ya existe
    if manager.check_image_exists():
        # Seleccionarla automáticamente
        config_store = get_config_store()
        config_store.set('selected_docker_image', DEFAULT_TERRAFORM_IMAGE)
        
        return {
            "status": "already_exists",
            "message": f"La imagen {DEFAULT_TERRAFORM_IMAGE} ya existe",
            "info": manager.get_image_info()
        }
    
    # Construir la imagen (con contenido opcional)
    result = manager.build_image(dockerfile_content=dockerfile_content)
    
    if result["status"] == "error":
        raise HTTPException(status_code=500, detail=result["message"])
    
    # Seleccionarla automáticamente si se construyó exitosamente
    if result["status"] == "success":
        config_store = get_config_store()
        config_store.set('selected_docker_image', DEFAULT_TERRAFORM_IMAGE)
    
    return result

@router.post("/rebuild")
def rebuild_image(force: bool = False, dockerfile_content: str = Body(None, embed=True)):
    """
    Reconstruye la imagen Docker (elimina y construye de nuevo).
    
    Args:
        force: Si es True, elimina la imagen existente antes de construir
        dockerfile_content: Contenido opcional del Dockerfile como string
    """
    manager = ImageManager(DEFAULT_TERRAFORM_IMAGE)
    
    # Eliminar imagen existente si force=True
    if force and manager.check_image_exists():
        remove_result = manager.remove_image(force=True)
        if remove_result["status"] == "error":
            raise HTTPException(status_code=500, detail=remove_result["message"])
    
    # Construir la imagen (con contenido opcional)
    result = manager.build_image(dockerfile_content=dockerfile_content)
    
    if result["status"] == "error":
        raise HTTPException(status_code=500, detail=result["message"])
    
    return result

@router.delete("/")
def delete_image():
    """Elimina la imagen Docker."""
    manager = ImageManager(DEFAULT_TERRAFORM_IMAGE)
    
    if not manager.check_image_exists():
        raise HTTPException(status_code=404, detail="Imagen no encontrada")
    
    result = manager.remove_image(force=True)
    
    if result["status"] == "error":
        raise HTTPException(status_code=500, detail=result["message"])
    
    return result



from fastapi import APIRouter, HTTPException
from terratest.constants import OUTPUTS_DIR

router = APIRouter()

@router.get("/api/jobs/{job_id}/files")
def list_job_files(job_id: str):
    folder = OUTPUTS_DIR / job_id
    if not folder.exists():
        raise HTTPException(status_code=404, detail="Job not found")

    return {
        "job_id": job_id,
        "files": sorted([f.name for f in folder.iterdir() if f.is_file()])
    }


from fastapi import APIRouter, HTTPException
from terratest.constants import OUTPUTS_DIR

router = APIRouter()

@router.get("/api/jobs/{job_id}/files/{filename}")
def read_job_file(job_id: str, filename: str):
    file_path = OUTPUTS_DIR / job_id / filename

    if not file_path.exists():
        raise HTTPException(status_code=404, detail="File not found")

    # Intentar leer con diferentes encodings
    encodings_to_try = ['utf-8', 'utf-8-sig', 'latin-1', 'cp1252']
    content = None
    
    for encoding in encodings_to_try:
        try:
            content = file_path.read_text(encoding=encoding)
            break
        except UnicodeDecodeError:
            continue
    
    if content is None:
        # Si ninguno funciona, leer como binario y decodificar con errors='replace'
        content = file_path.read_text(encoding='utf-8', errors='replace')
    
    return {"content": content}


import json
from pathlib import Path


class TerraformPlanParser:
    """Convierte plan.json en datos estructurados útiles."""

    def load_plan(self, plan_path: Path) -> dict:
        if not plan_path.exists():
            raise FileNotFoundError(f"El archivo {plan_path} no existe")

        with plan_path.open("r", encoding="utf-8") as f:
            data = json.load(f)

        return data

    def get_summary(self, plan: dict) -> dict:
        """Retorna:
        - recursos a crear
        - actualizar
        - eliminar
        """

        resource_changes = plan.get("resource_changes", [])
        summary = {"add": 0, "change": 0, "destroy": 0}

        for rc in resource_changes:
            action = rc.get("change", {}).get("actions", [])
            if "create" in action:
                summary["add"] += 1
            if "update" in action:
                summary["change"] += 1
            if "delete" in action:
                summary["destroy"] += 1

        return summary

    def get_resources(self, plan: dict) -> list:
        """Retorna lista de recursos con tipo, nombre y acciones."""
        result = []
        for rc in plan.get("resource_changes", []):
            result.append({
                "type": rc.get("type"),
                "name": rc.get("name"),
                "actions": rc.get("change", {}).get("actions", []),
            })
        return result



from pathlib import Path

APP_NAME = "terratest"

CONFIG_DIR = Path.home() / ".terratest"
JOBS_DIR = CONFIG_DIR / "jobs"
WORKSPACE_DIR = JOBS_DIR / "workspace"
OUTPUTS_DIR = JOBS_DIR / "outputs"
LOGS_DIR = JOBS_DIR / "logs"

# Por si luego quieres cambiar la imagen de Docker
DEFAULT_TERRAFORM_IMAGE = "terratest/terraform:latest"


from fastapi import APIRouter, HTTPException, Query
from pathlib import Path
from typing import Optional
import shutil
from terratest.core.executor import JobExecutor
from terratest.constants import OUTPUTS_DIR
from terratest.utils.config_store import get_config_store

router = APIRouter()

@router.post("/")
def run_job(
    module_path: str,
    enable_ssh: bool = Query(False, description="Habilitar autenticación SSH para Git"),
    ssh_key_path: Optional[str] = Query(None, description="Path personalizado a la clave SSH"),
    tf_cloud_token: Optional[str] = Query(None, description="Token de Terraform Cloud"),
    tf_cloud_org: Optional[str] = Query(None, description="Organización de Terraform Cloud"),
    tf_cloud_workspace: Optional[str] = Query(None, description="Workspace de Terraform Cloud"),
    aws_access_key_id: Optional[str] = Query(None, description="AWS Access Key ID"),
    aws_secret_access_key: Optional[str] = Query(None, description="AWS Secret Access Key"),
    aws_session_token: Optional[str] = Query(None, description="AWS Session Token (opcional)"),
):
    """
    Ejecutar un job de Terraform con configuración opcional:
    - SSH: Para repositorios privados de Git
    - Terraform Cloud: Para remote state y colaboración
    - AWS: Credenciales para proveedores de AWS
    """
    # Guardar configuración en store temporal
    config_store = get_config_store()
    config_data = {
        'enable_ssh': enable_ssh,
        'ssh_key_path': ssh_key_path,
        'tf_cloud_token': tf_cloud_token,
        'tf_cloud_org': tf_cloud_org,
        'tf_cloud_workspace': tf_cloud_workspace,
        'aws_access_key_id': aws_access_key_id,
        'aws_secret_access_key': aws_secret_access_key,
        'aws_session_token': aws_session_token,
    }
    config_store.update(config_data)
    
    executor = JobExecutor()
    return executor.execute_job(
        module_path=module_path,
        enable_ssh=enable_ssh,
        ssh_key_path=ssh_key_path,
        tf_cloud_token=tf_cloud_token,
        tf_cloud_org=tf_cloud_org,
        tf_cloud_workspace=tf_cloud_workspace,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
    )

@router.get("/")
def list_jobs():
    jobs = []
    
    if not OUTPUTS_DIR.exists():
        return jobs

    for job_dir in OUTPUTS_DIR.iterdir():
        if job_dir.is_dir():
            files = [f.name for f in job_dir.iterdir() if f.is_file()]
            jobs.append({
                "job_id": job_dir.name,
                "files": files,
                "modified_time": job_dir.stat().st_mtime
            })

    # Ordenar por fecha de modificación (más reciente primero)
    jobs.sort(key=lambda x: x["modified_time"], reverse=True)
    
    # Remover el campo modified_time antes de retornar
    for job in jobs:
        del job["modified_time"]

    return jobs

@router.delete("/all")
def delete_all_jobs():
    """Eliminar todos los jobs"""
    if not OUTPUTS_DIR.exists():
        return {"message": "No hay jobs para eliminar"}
    
    deleted_count = 0
    errors = []
    
    for job_dir in OUTPUTS_DIR.iterdir():
        if job_dir.is_dir():
            try:
                shutil.rmtree(job_dir)
                deleted_count += 1
            except Exception as e:
                errors.append(f"{job_dir.name}: {str(e)}")
    
    if errors:
        return {
            "message": f"Se eliminaron {deleted_count} jobs con {len(errors)} errores",
            "deleted": deleted_count,
            "errors": errors
        }
    
    return {
        "message": f"Se eliminaron {deleted_count} jobs correctamente",
        "deleted": deleted_count
    }

@router.delete("/{job_id}")
def delete_job(job_id: str):
    """Eliminar un job específico"""
    job_dir = OUTPUTS_DIR / job_id
    
    if not job_dir.exists() or not job_dir.is_dir():
        raise HTTPException(status_code=404, detail="Job no encontrado")
    
    try:
        shutil.rmtree(job_dir)
        return {"message": f"Job {job_id} eliminado correctamente"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error al eliminar job: {str(e)}")


from pathlib import Path
import shutil
import os

from terratest.constants import CONFIG_DIR, JOBS_DIR, WORKSPACE_DIR, OUTPUTS_DIR, LOGS_DIR


def ensure_base_dirs() -> None:
    """
    Crea la estructura base:
    ~/.terratest/
        jobs/workspace/
        jobs/outputs/
        jobs/logs/
    """
    for path in [CONFIG_DIR, JOBS_DIR, WORKSPACE_DIR, OUTPUTS_DIR, LOGS_DIR]:
        path.mkdir(parents=True, exist_ok=True)


def ensure_dir(path: Path) -> Path:
    path.mkdir(parents=True, exist_ok=True)
    return path


def copy_tree(src: Path, dst: Path) -> None:
    """
    Copia el contenido de src a dst (como cp -r).
    Si dst existe, se mezcla (no se borra).
    """
    src = src.resolve()
    dst = dst.resolve()

    if not src.exists() or not src.is_dir():
        raise FileNotFoundError(f"Directorio de origen no válido: {src}")

    dst.mkdir(parents=True, exist_ok=True)

    for item in src.iterdir():
        target = dst / item.name
        if item.is_dir():
            shutil.copytree(item, target, dirs_exist_ok=True)
        else:
            shutil.copy2(item, target)


def safe_remove(path: Path) -> None:
    """
    Borra archivos o directorios de forma segura.
    """
    if not path.exists():
        return

    if path.is_dir():
        shutil.rmtree(path)
    else:
        path.unlink()


def get_job_workspace_dir(job_id: str) -> Path:
    return WORKSPACE_DIR / job_id


def get_job_output_dir(job_id: str) -> Path:
    return OUTPUTS_DIR / job_id


def get_job_log_file(job_id: str) -> Path:
    return LOGS_DIR / f"{job_id}.log"



import docker
from docker.models.containers import Container
from pathlib import Path
from typing import List, Dict, Optional

from terratest.constants import DEFAULT_TERRAFORM_IMAGE


class DockerManager:
    """
    Manejo de contenedores Terraform con soporte SSH real.
    """

    def __init__(self, image: str = DEFAULT_TERRAFORM_IMAGE):
        self.client = docker.from_env()
        self.image = image

    def create_container(
        self,
        job_id: str,
        workspace: Path,
        output: Path,
        work: Path,
        ssh_home: Optional[Path] = None,
        enable_ssh: bool = False,
        env: Optional[Dict[str, str]] = None,
        tf_cloud_token: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_session_token: Optional[str] = None,
    ) -> Container:

        volumes = {
            str(workspace): {"bind": "/workspace", "mode": "ro"},
            str(output):    {"bind": "/output", "mode": "rw"},
            str(work):      {"bind": "/work", "mode": "rw"},
        }

        # Variables de entorno
        env = env or {}

        # ← SSH support (solo si está habilitado)
        if enable_ssh and ssh_home and ssh_home.exists():
            volumes[str(ssh_home)] = {"bind": "/ssh-home", "mode": "rw"}
            env["GIT_SSH_COMMAND"] = "ssh -i /ssh-home/id_rsa -o StrictHostKeyChecking=no"
        
        # ← Terraform Cloud support
        if tf_cloud_token:
            env["TF_TOKEN_app_terraform_io"] = tf_cloud_token
        
        # ← AWS Credentials support
        if aws_access_key_id:
            env["AWS_ACCESS_KEY_ID"] = aws_access_key_id
        if aws_secret_access_key:
            env["AWS_SECRET_ACCESS_KEY"] = aws_secret_access_key
        if aws_session_token:
            env["AWS_SESSION_TOKEN"] = aws_session_token

        container = self.client.containers.create(
            image=self.image,
            name=f"terratest-{job_id}",
            entrypoint=["/bin/sh", "-c", "tail -f /dev/null"],
            volumes=volumes,
            environment=env,
            network_mode="bridge",
            tty=True,
            stdin_open=True,
            auto_remove=True,
        )

        return container

    def start_container(self, container: Container):
        container.start()

    def exec(self, container: Container, cmd: List[str]) -> str:
        exec_id = self.client.api.exec_create(container.id, cmd)
        output = self.client.api.exec_start(exec_id).decode("utf-8")
        return output

    def stream_exec(self, container, cmd: list):
        exec_id = self.client.api.exec_create(container.id, cmd, tty=True)
        stream = self.client.api.exec_start(exec_id, stream=True)
        for chunk in stream:
            yield chunk.decode("utf-8", errors="ignore")

    def stop_container(self, container: Container):
        try:
            container.stop()
        except Exception:
            pass

    def cleanup_container(self, container: Container):
        try:
            container.remove(force=True)
        except Exception:
            pass



from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles
from fastapi.responses import FileResponse
from pathlib import Path
from terratest.web.routes.jobs import router as jobs_router
from terratest.web.routes.job_files import router as job_files_router
from terratest.web.routes.job_file_read import router as job_file_read_router
from terratest.web.routes.filesystem import router as filesystem_router
from terratest.web.routes.docker_image import router as docker_image_router

app = FastAPI(title="Terratest WebUI")

STATIC_DIR = Path(__file__).parent / "static"

# Servir carpeta estática
app.mount("/static", StaticFiles(directory=STATIC_DIR), name="static")

@app.get("/")
def root():
    index = STATIC_DIR / "index.html"
    return FileResponse(index)

app.include_router(jobs_router, prefix="/api/jobs")
app.include_router(job_files_router)
app.include_router(job_file_read_router)
app.include_router(filesystem_router)
app.include_router(docker_image_router, prefix="/api/docker-image")

@app.get("/health")
def health():
    return {"status": "ok"}




import typer
from rich import print

from terratest.core.executor import JobExecutor


def run_command(module_path: str):
    print("[bold cyan]🔧 Terratest — Ejecutando módulo[/bold cyan]")
    print(f"📁 Módulo: {module_path}")

    executor = JobExecutor()
    result = executor.execute_job(
        module_path=module_path,
        run_init=True,
        run_plan=True,
        run_apply=False,
    )

    job_id = result.get("job_id")
    status = result.get("status")

    if job_id:
        print(f"🆔 Job ID: [bold]{job_id}[/bold]")
    print(f"📊 Status: [bold]{status}[/bold]")

    # Info útil en FASE 1
    workspace = result.get("workspace_dir")
    output_dir = result.get("output_dir")
    if workspace:
        print(f"📂 Workspace: {workspace}")
    if output_dir:
        print(f"📂 Outputs:   {output_dir}")

    if "error" in result:
        print(f"[red]❌ Error:[/red] {result['error']}")
    else:
        print("[green]✔ Workspace preparado correctamente[/green]")


from pathlib import Path
from typing import Tuple, List


class ValidationError(Exception):
    pass


def validate_module_path(module_path: str | Path) -> Path:
    """
    Normaliza y valida el path del módulo.
    - Debe existir
    - Debe ser directorio
    - Debe contener al menos un archivo .tf
    """
    path = Path(module_path).expanduser().resolve()

    if not path.exists():
        raise ValidationError(f"El módulo no existe: {path}")

    if not path.is_dir():
        raise ValidationError(f"El módulo no es un directorio: {path}")

    tf_files = list(path.glob("*.tf"))
    if not tf_files:
        raise ValidationError(f"El módulo no contiene archivos .tf: {path}")

    return path


def validate_terraform_files(path: Path) -> List[Path]:
    """
    Devuelve la lista de archivos .tf válidos en el módulo.
    Lanza error si no hay ninguno.
    """
    tf_files = sorted(path.glob("*.tf"))
    if not tf_files:
        raise ValidationError(f"No se encontraron archivos .tf en {path}")
    return tf_files



"""
Rutas para exploración del sistema de archivos
"""
from fastapi import APIRouter, HTTPException, Query
from pathlib import Path
from typing import List
import os

router = APIRouter(prefix="/api/filesystem", tags=["filesystem"])


@router.get("/project-dir")
async def get_project_directory():
    """
    Devuelve el directorio raíz del proyecto actual
    """
    try:
        # Obtener el directorio actual de trabajo
        project_dir = Path.cwd()
        return {
            "project_dir": str(project_dir),
            "normalized": str(project_dir).replace('\\', '/')
        }
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error al obtener directorio del proyecto: {str(e)}")


@router.get("/list")
async def list_directory(path: str = Query(".", description="Path del directorio a listar")):
    """
    Lista los directorios dentro de un path dado.
    Soporta paths relativos y absolutos.
    """
    try:
        # Convertir a Path y resolver
        if path == ".":
            dir_path = Path.cwd()
        else:
            dir_path = Path(path)
            # Si no es absoluto, hacerlo relativo al directorio actual
            if not dir_path.is_absolute():
                dir_path = Path.cwd() / dir_path
            dir_path = dir_path.resolve()
        
        # Verificar que existe y es un directorio
        if not dir_path.exists():
            raise HTTPException(status_code=404, detail=f"El directorio no existe: {path}")
        
        if not dir_path.is_dir():
            raise HTTPException(status_code=400, detail=f"La ruta no es un directorio: {path}")
        
        # Listar solo directorios (excluir archivos)
        directories = []
        try:
            for item in sorted(dir_path.iterdir()):
                # Solo incluir directorios, excluir algunos especiales
                if item.is_dir():
                    # Excluir directorios ocultos en Linux/Mac y algunos especiales
                    if item.name.startswith('.') and item.name not in ['.']:
                        continue
                    if item.name.startswith('__'):
                        continue
                    directories.append(item.name)
        except PermissionError:
            raise HTTPException(status_code=403, detail="Sin permisos para leer el directorio")
        
        # Información del path padre
        parent_path = None
        if dir_path.parent != dir_path:  # No estamos en la raíz
            parent_path = str(dir_path.parent)
        
        return {
            "path": str(dir_path),
            "normalized_path": str(dir_path).replace('\\', '/'),
            "parent": parent_path,
            "directories": directories,
            "count": len(directories)
        }
        
    except HTTPException:
        raise
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error al listar directorio: {str(e)}")



import typer
import uvicorn

def web_command(port: int = 8765):
    typer.echo(f"🌐 Iniciando Terratest Web UI en http://localhost:{port}")
    uvicorn.run("terratest.web.app:app", host="0.0.0.0", port=port, reload=True)



"""
Sistema de persistencia temporal para configuraciones de Terratest.
Las configuraciones se guardan mientras el servicio está activo.
"""
import json
from pathlib import Path
from typing import Dict, Any, Optional
from datetime import datetime


class ConfigStore:
    """Almacena configuración temporal mientras el servicio está activo."""
    
    def __init__(self, store_path: Optional[Path] = None):
        if store_path is None:
            store_path = Path.home() / ".terratest" / "temp_config.json"
        
        self.store_path = store_path
        self.store_path.parent.mkdir(parents=True, exist_ok=True)
        self._ensure_file()
    
    def _ensure_file(self):
        """Crea el archivo si no existe."""
        if not self.store_path.exists():
            self.save({})
    
    def load(self) -> Dict[str, Any]:
        """Carga la configuración desde el archivo."""
        try:
            with open(self.store_path, 'r', encoding='utf-8') as f:
                return json.load(f)
        except (json.JSONDecodeError, FileNotFoundError):
            return {}
    
    def save(self, config: Dict[str, Any]):
        """Guarda la configuración en el archivo."""
        config['_updated_at'] = datetime.utcnow().isoformat() + 'Z'
        with open(self.store_path, 'w', encoding='utf-8') as f:
            json.dump(config, f, indent=2, ensure_ascii=False)
    
    def get(self, key: str, default: Any = None) -> Any:
        """Obtiene un valor específico de la configuración."""
        config = self.load()
        return config.get(key, default)
    
    def set(self, key: str, value: Any):
        """Establece un valor específico en la configuración."""
        config = self.load()
        config[key] = value
        self.save(config)
    
    def update(self, updates: Dict[str, Any]):
        """Actualiza múltiples valores a la vez."""
        config = self.load()
        config.update(updates)
        self.save(config)
    
    def delete(self, key: str):
        """Elimina un valor específico."""
        config = self.load()
        if key in config:
            del config[key]
            self.save(config)
    
    def clear(self):
        """Limpia toda la configuración."""
        self.save({})
    
    def cleanup(self):
        """Elimina el archivo de configuración temporal."""
        if self.store_path.exists():
            self.store_path.unlink()


# Instancia global del store
_config_store: Optional[ConfigStore] = None


def get_config_store() -> ConfigStore:
    """Obtiene la instancia global del config store."""
    global _config_store
    if _config_store is None:
        _config_store = ConfigStore()
    return _config_store



"""
Gestor de imágenes Docker para Terratest.
Verifica y construye la imagen de Terraform si no existe.
"""
import docker
from pathlib import Path
from typing import Dict, Any, Optional
import logging

logger = logging.getLogger(__name__)


class ImageManager:
    """Gestiona la imagen Docker de Terraform."""
    
    def __init__(self, image_name: str = "terratest/terraform:latest"):
        self.client = docker.from_env()
        self.image_name = image_name
    
    def check_image_exists(self) -> bool:
        """Verifica si la imagen existe localmente."""
        try:
            self.client.images.get(self.image_name)
            return True
        except docker.errors.ImageNotFound:
            return False
        except Exception as e:
            logger.error(f"Error al verificar imagen: {e}")
            return False
    
    def list_all_images(self) -> list[Dict[str, Any]]:
        """Lista todas las imágenes Docker disponibles localmente."""
        try:
            images = self.client.images.list()
            result = []
            
            for image in images:
                # Obtener tags (nombres) de la imagen
                tags = image.tags if image.tags else ["<none>:<none>"]
                
                for tag in tags:
                    result.append({
                        "id": image.short_id.replace('sha256:', ''),
                        "tag": tag,
                        "created": image.attrs.get('Created', 'Unknown'),
                        "size_mb": round(image.attrs.get('Size', 0) / (1024 * 1024), 2)
                    })
            
            # Ordenar por fecha de creación (más reciente primero)
            result.sort(key=lambda x: x['created'], reverse=True)
            
            return result
        except Exception as e:
            logger.error(f"Error al listar imágenes: {e}")
            return []
    
    def get_dockerfile_path(self) -> Optional[Path]:
        """Obtiene la ruta al Dockerfile."""
        # Desde terratest/core/image_manager.py subir a terratest/ y luego a docker/
        current_file = Path(__file__)
        terratest_root = current_file.parent.parent
        dockerfile = terratest_root.parent / "docker" / "Dockerfile.terraform"
        
        if dockerfile.exists():
            return dockerfile
        
        # Intentar buscar en otras ubicaciones comunes
        alternatives = [
            Path.cwd() / "docker" / "Dockerfile.terraform",
            Path.cwd() / "Dockerfile.terraform",
        ]
        
        for alt in alternatives:
            if alt.exists():
                return alt
        
        return None
    
    def build_image(self, dockerfile_path: Optional[Path] = None, dockerfile_content: Optional[str] = None) -> Dict[str, Any]:
        """
        Construye la imagen Docker.
        
        Args:
            dockerfile_path: Ruta al Dockerfile (opcional)
            dockerfile_content: Contenido del Dockerfile como string (opcional)
        
        Returns:
            Dict con status, message, y logs de la construcción
        """
        try:
            build_logs = []
            
            # Si se proporciona contenido directamente, usarlo
            if dockerfile_content:
                logger.info(f"Construyendo imagen {self.image_name} desde contenido proporcionado...")
                
                import io
                import tarfile
                
                # Crear un contexto tar en memoria con el Dockerfile
                tar_stream = io.BytesIO()
                tar = tarfile.open(fileobj=tar_stream, mode='w')
                
                # Agregar el Dockerfile al tar
                dockerfile_data = dockerfile_content.encode('utf-8')
                tarinfo = tarfile.TarInfo(name='Dockerfile')
                tarinfo.size = len(dockerfile_data)
                tar.addfile(tarinfo, io.BytesIO(dockerfile_data))
                tar.close()
                
                # Volver al inicio del stream
                tar_stream.seek(0)
                
                image, logs = self.client.images.build(
                    fileobj=tar_stream,
                    tag=self.image_name,
                    rm=True,
                    forcerm=True,
                    custom_context=True,
                )
            
            # Si no hay contenido, buscar archivo
            else:
                if dockerfile_path is None:
                    dockerfile_path = self.get_dockerfile_path()
                
                if dockerfile_path is None or not dockerfile_path.exists():
                    # Proporcionar Dockerfile por defecto
                    default_dockerfile = """FROM hashicorp/terraform:latest

# Instalar herramientas adicionales
RUN apk add --no-cache git openssh-client

# Configurar SSH
RUN mkdir -p /root/.ssh && chmod 700 /root/.ssh

WORKDIR /workspace
"""
                    logger.info(f"No se encontró Dockerfile, usando configuración por defecto...")
                    
                    import io
                    import tarfile
                    
                    # Crear contexto tar con Dockerfile por defecto
                    tar_stream = io.BytesIO()
                    tar = tarfile.open(fileobj=tar_stream, mode='w')
                    
                    dockerfile_data = default_dockerfile.encode('utf-8')
                    tarinfo = tarfile.TarInfo(name='Dockerfile')
                    tarinfo.size = len(dockerfile_data)
                    tar.addfile(tarinfo, io.BytesIO(dockerfile_data))
                    tar.close()
                    
                    tar_stream.seek(0)
                    
                    image, logs = self.client.images.build(
                        fileobj=tar_stream,
                        tag=self.image_name,
                        rm=True,
                        forcerm=True,
                        custom_context=True,
                    )
                else:
                    # Usar archivo encontrado
                    context_path = dockerfile_path.parent.parent
                    dockerfile_rel = dockerfile_path.relative_to(context_path)
                    dockerfile_rel_str = str(dockerfile_rel).replace('\\', '/')
                    
                    logger.info(f"Construyendo imagen {self.image_name}...")
                    logger.info(f"Context: {context_path}")
                    logger.info(f"Dockerfile: {dockerfile_rel_str}")
                    
                    image, logs = self.client.images.build(
                        path=str(context_path),
                        dockerfile=dockerfile_rel_str,
                        tag=self.image_name,
                        rm=True,
                        forcerm=True,
                    )
            
            # Capturar logs
            for chunk in logs:
                if 'stream' in chunk:
                    log_line = chunk['stream'].strip()
                    if log_line:
                        build_logs.append(log_line)
                        logger.info(log_line)
                elif 'error' in chunk:
                    error_msg = chunk['error'].strip()
                    build_logs.append(f"ERROR: {error_msg}")
                    logger.error(error_msg)
            
            return {
                "status": "success",
                "message": f"Imagen {self.image_name} construida exitosamente",
                "image_id": image.id,
                "logs": build_logs
            }
            
        except docker.errors.BuildError as e:
            logger.error(f"Error al construir imagen: {e}")
            return {
                "status": "error",
                "message": f"Error al construir imagen: {str(e)}",
                "logs": [str(e)]
            }
        except Exception as e:
            logger.error(f"Error inesperado: {e}")
            return {
                "status": "error",
                "message": f"Error inesperado: {str(e)}",
                "logs": [str(e)]
            }
    
    def get_image_info(self) -> Optional[Dict[str, Any]]:
        """Obtiene información de la imagen si existe."""
        try:
            image = self.client.images.get(self.image_name)
            return {
                "id": image.id,
                "short_id": image.short_id,
                "tags": image.tags,
                "created": image.attrs.get('Created', 'Unknown'),
                "size": image.attrs.get('Size', 0),
                "size_mb": round(image.attrs.get('Size', 0) / (1024 * 1024), 2)
            }
        except docker.errors.ImageNotFound:
            return None
        except Exception as e:
            logger.error(f"Error al obtener info de imagen: {e}")
            return None
    
    def remove_image(self, force: bool = False) -> Dict[str, Any]:
        """Elimina la imagen Docker."""
        try:
            self.client.images.remove(self.image_name, force=force)
            return {
                "status": "success",
                "message": f"Imagen {self.image_name} eliminada"
            }
        except docker.errors.ImageNotFound:
            return {
                "status": "error",
                "message": "Imagen no encontrada"
            }
        except Exception as e:
            return {
                "status": "error",
                "message": f"Error al eliminar imagen: {str(e)}"
            }



import logging
from pathlib import Path
from typing import Optional

from terratest.utils.filesystem import ensure_base_dirs, get_job_log_file


def setup_root_logger(level: int = logging.INFO) -> None:
    """
    Logger base de la app (CLI/Web).
    """
    logging.basicConfig(
        level=level,
        format="%(asctime)s [%(levelname)s] %(name)s: %(message)s",
    )


def get_job_logger(job_id: str) -> logging.Logger:
    """
    Crea un logger específico para un job que escribe a:
    ~/.terratest/jobs/logs/{job_id}.log
    """
    ensure_base_dirs()
    logger_name = f"terratest.job.{job_id}"
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.INFO)

    # Evitar añadir múltiples handlers si se llama varias veces
    if not logger.handlers:
        log_file: Path = get_job_log_file(job_id)
        fh = logging.FileHandler(log_file, encoding="utf-8")
        fmt = logging.Formatter(
            "%(asctime)s [%(levelname)s] %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
        fh.setFormatter(fmt)
        logger.addHandler(fh)

    return logger



from datetime import datetime
import uuid
import json
from pathlib import Path
from typing import Dict, Any, Optional

from terratest.core.docker_manager import DockerManager
from terratest.core.validation import validate_module_path, validate_terraform_files, ValidationError
from terratest.utils.filesystem import (
    ensure_base_dirs,
    get_job_workspace_dir,
    get_job_output_dir,
    copy_tree,
)
from terratest.utils.logging import get_job_logger
from terratest.utils.config_store import get_config_store
from terratest.constants import DEFAULT_TERRAFORM_IMAGE


class JobExecutor:
    """FASE 1–4: Docker + Terraform real + Parser con SSH integrado."""

    def __init__(self) -> None:
        ensure_base_dirs()
        
        # Obtener imagen seleccionada del config store
        config_store = get_config_store()
        selected_image = config_store.get('selected_docker_image', DEFAULT_TERRAFORM_IMAGE)
        
        self.docker = DockerManager(image=selected_image)

    def _generate_job_id(self) -> str:
        return f"job-{uuid.uuid4().hex[:8]}"

    def _stream_and_log(self, logger, container, cmd: list, log_file: Path):
        with log_file.open("a", encoding="utf-8") as f:
            for line in self.docker.stream_exec(container, cmd):
                clean = line.rstrip()
                logger.info(clean)
                f.write(clean + "\n")

    def _parse_outputs_from_plan(self, plan_data: dict) -> dict:
        outputs = {}
        output_changes = plan_data.get("output_changes", {})
        for key, item in output_changes.items():
            outputs[key] = item.get("after")
        return outputs

    def _parse_plan_json(self, plan_path: Path) -> Dict[str, Any]:
        if not plan_path.exists():
            return {"summary": None, "resources": []}

        raw = plan_path.read_text(encoding="utf-8").strip()
        if not raw:
            return {"summary": None, "resources": []}

        data = json.loads(raw)
        changes = data.get("resource_changes", [])

        summary = {"add": 0, "change": 0, "destroy": 0}
        resources = []

        for rc in changes:
            actions = rc.get("change", {}).get("actions", [])
            if "create" in actions:
                summary["add"] += 1
            if "update" in actions:
                summary["change"] += 1
            if "delete" in actions:
                summary["destroy"] += 1

            resources.append({
                "type": rc.get("type"),
                "name": rc.get("name"),
                "actions": actions,
            })

        return {"summary": summary, "resources": resources}

    def execute_job(
        self,
        module_path: str | Path,
        secrets: Optional[Dict[str, str]] = None,
        run_init: bool = True,
        run_plan: bool = True,
        run_apply: bool = False,
        timeout: int = 600,
        enable_ssh: bool = False,
        ssh_key_path: Optional[str] = None,
        tf_cloud_token: Optional[str] = None,
        tf_cloud_org: Optional[str] = None,
        tf_cloud_workspace: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_session_token: Optional[str] = None,
    ) -> Dict[str, Any]:

        started_at = datetime.utcnow()
        job_id = self._generate_job_id()
        logger = get_job_logger(job_id)

        container = None

        try:
            # --------------------- FASE 1 ---------------------
            module_path = validate_module_path(module_path)
            validate_terraform_files(module_path)

            job_workspace = get_job_workspace_dir(job_id)
            job_output = get_job_output_dir(job_id)
            copy_tree(module_path, job_workspace)

            job_work = job_output / "work"
            job_work.mkdir(parents=True, exist_ok=True)

            log_file = job_output / "terraform.log"

            # Configuración SSH
            env = {}
            ssh_home = None
            
            if enable_ssh:
                if ssh_key_path:
                    ssh_home = Path(ssh_key_path).parent
                else:
                    ssh_home = Path.home() / ".ssh"
                
                if ssh_home.exists():
                    logger.info(f"SSH habilitado: {ssh_home}")
                    env["GIT_SSH_COMMAND"] = (
                        "ssh -i /ssh-home/id_rsa "
                        "-o StrictHostKeyChecking=no "
                        "-o UserKnownHostsFile=/ssh-home/known_hosts"
                    )
                else:
                    logger.warning(f"SSH habilitado pero directorio no existe: {ssh_home}")
                    ssh_home = None
            else:
                logger.info("SSH deshabilitado")
            
            # Configuración Terraform Cloud
            if tf_cloud_token:
                logger.info("Terraform Cloud habilitado")
                env["TF_TOKEN_app_terraform_io"] = tf_cloud_token
                
                if tf_cloud_org and tf_cloud_workspace:
                    logger.info(f"Workspace Cloud: {tf_cloud_org}/{tf_cloud_workspace}")
            
            # Configuración AWS
            if aws_access_key_id:
                logger.info("AWS credentials configuradas")
                env["AWS_ACCESS_KEY_ID"] = aws_access_key_id
                
                if aws_secret_access_key:
                    env["AWS_SECRET_ACCESS_KEY"] = aws_secret_access_key
                
                if aws_session_token:
                    env["AWS_SESSION_TOKEN"] = aws_session_token
                    logger.info("AWS session token incluido")

            # --------------------- FASE 2 ---------------------
            container = self.docker.create_container(
                job_id=job_id,
                workspace=job_workspace,
                output=job_output,
                work=job_work,
                ssh_home=ssh_home,
                enable_ssh=enable_ssh,
                env=env,
                tf_cloud_token=tf_cloud_token,
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                aws_session_token=aws_session_token,
            )
            self.docker.start_container(container)

            # Fix de permisos SSH — solo si está habilitado
            if enable_ssh and ssh_home:
                logger.info("Configurando permisos SSH en container...")
                self.docker.exec(container, ["/bin/sh", "-c", "chmod 600 /ssh-home/id_rsa || true"])
                self.docker.exec(container, ["/bin/sh", "-c", "chmod 644 /ssh-home/id_rsa.pub || true"])
                self.docker.exec(container, ["/bin/sh", "-c", "touch /ssh-home/known_hosts"])
                self.docker.exec(container, ["/bin/sh", "-c", "chmod 644 /ssh-home/known_hosts"])

            tf_version = self.docker.exec(container, ["terraform", "--version"])

            # --------------------- FASE 3 ---------------------
            if run_init:
                self._stream_and_log(
                    logger, container,
                    ["/bin/sh", "-c",
                     "cd /work && terraform init -input=false -no-color -lock=false -from-module=/workspace"],
                    log_file,
                )

            if run_plan:
                self._stream_and_log(
                    logger, container,
                    ["/bin/sh", "-c",
                     "cd /work && terraform plan -input=false -no-color -lock=false -out=/work/tfplan"],
                    log_file,
                )

                self._stream_and_log(
                    logger, container,
                    ["/bin/sh", "-c",
                     "cd /work && terraform show -json /work/tfplan | tee /output/plan.json"],
                    log_file,
                )

            # --------------------- OUTPUTS ---------------------
            outputs_raw = self.docker.exec(
                container,
                ["/bin/sh", "-c", "cd /work && (terraform output -json || echo '{}')"]
            )
            (job_output / "outputs.json").write_text(outputs_raw, encoding="utf-8")

            outputs = json.loads(outputs_raw.strip() or "{}")

            if not outputs:
                try:
                    plan_data = json.loads((job_output / "plan.json").read_text(encoding="utf-8"))
                    outputs = self._parse_outputs_from_plan(plan_data)
                except:
                    outputs = {}

            # --------------------- FASE 4 ---------------------
            parsed = self._parse_plan_json(job_output / "plan.json")

            finished_at = datetime.utcnow()

            return {
                "job_id": job_id,
                "status": "terraform_complete",
                "workspace_dir": str(job_workspace),
                "output_dir": str(job_output),
                "terraform_version": tf_version.strip(),
                "started_at": started_at.isoformat() + "Z",
                "finished_at": finished_at.isoformat() + "Z",
                "duration_seconds": (finished_at - started_at).total_seconds(),
                "plan_summary": parsed["summary"],
                "resources": parsed["resources"],
                "outputs": outputs,
            }

        except Exception as ex:
            return {"job_id": job_id, "status": "error", "error": str(ex)}

        finally:
            if container:
                try: self.docker.stop_container(container)
                except: pass
                try: self.docker.cleanup_container(container)
                except: pass



from fastapi import APIRouter
from pathlib import Path
from terratest.constants import OUTPUTS_DIR

router = APIRouter()

@router.get("/")
def list_jobs():
    jobs = []

    for job_dir in OUTPUTS_DIR.iterdir():
        if job_dir.is_dir():
            files = [f.name for f in job_dir.iterdir() if f.is_file()]
            jobs.append({
                "job_id": job_dir.name,
                "files": files
            })

    return jobs


import typer
from terratest.cli.commands.run import run_command
from terratest.cli.commands.web import web_command

app = typer.Typer(help="CLI de Terratest")

app.command("run")(run_command)
app.command("web")(web_command)


from fastapi import APIRouter, HTTPException
from pathlib import Path
import shutil
from terratest.constants import OUTPUTS_DIR

router = APIRouter()

@router.delete("/{job_id}")
def delete_job(job_id: str):
    """Eliminar un job específico"""
    job_dir = OUTPUTS_DIR / job_id
    
    if not job_dir.exists() or not job_dir.is_dir():
        raise HTTPException(status_code=404, detail="Job no encontrado")
    
    try:
        shutil.rmtree(job_dir)
        return {"message": f"Job {job_id} eliminado correctamente"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error al eliminar job: {str(e)}")

@router.delete("/")
def delete_all_jobs():
    """Eliminar todos los jobs"""
    if not OUTPUTS_DIR.exists():
        return {"message": "No hay jobs para eliminar"}
    
    deleted_count = 0
    errors = []
    
    for job_dir in OUTPUTS_DIR.iterdir():
        if job_dir.is_dir():
            try:
                shutil.rmtree(job_dir)
                deleted_count += 1
            except Exception as e:
                errors.append(f"{job_dir.name}: {str(e)}")
    
    if errors:
        return {
            "message": f"Se eliminaron {deleted_count} jobs con {len(errors)} errores",
            "deleted": deleted_count,
            "errors": errors
        }
    
    return {
        "message": f"Se eliminaron {deleted_count} jobs correctamente",
        "deleted": deleted_count
    }
